import { FromSchema } from 'json-schema-to-ts';
import { GraphQLESLintRule } from '../../types.js';
import 'eslint';
import 'estree';
import 'graphql';
import 'graphql-config';
import '../../estree-converter/types.js';
import '../../siblings.js';
import '@graphql-tools/utils';

declare const schema: {
    readonly type: "array";
    readonly maxItems: 1;
    readonly items: {
        readonly type: "object";
        readonly additionalProperties: false;
        readonly properties: {
            readonly checkInputType: {
                readonly type: "boolean";
                readonly default: false;
                readonly description: "Check that the input type name follows the convention \\<mutationName>Input";
            };
            readonly caseSensitiveInputType: {
                readonly type: "boolean";
                readonly default: true;
                readonly description: "Allow for case discrepancies in the input type name";
            };
            readonly checkQueries: {
                readonly type: "boolean";
                readonly default: false;
                readonly description: "Apply the rule to Queries";
            };
            readonly checkMutations: {
                readonly type: "boolean";
                readonly default: true;
                readonly description: "Apply the rule to Mutations";
            };
        };
    };
};
type RuleOptions = FromSchema<typeof schema, {
    keepDefaultedPropertiesOptional: true;
}>;
declare const rule: GraphQLESLintRule<RuleOptions>;

export { type RuleOptions, rule };
